# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bl_make_null_box <- function(width_pt = 0, height_pt = 0) {
    .Call(`_gridtext_bl_make_null_box`, width_pt, height_pt)
}

bl_make_par_box <- function(node_list, vspacing_pt, width_policy = "native", hjust = NULL) {
    .Call(`_gridtext_bl_make_par_box`, node_list, vspacing_pt, width_policy, hjust)
}

bl_make_rect_box <- function(content, width_pt, height_pt, margin, padding, gp, content_hjust = 0, content_vjust = 1, width_policy = "fixed", height_policy = "fixed", r = 0) {
    .Call(`_gridtext_bl_make_rect_box`, content, width_pt, height_pt, margin, padding, gp, content_hjust, content_vjust, width_policy, height_policy, r)
}

bl_make_text_box <- function(label, gp, voff_pt = 0) {
    .Call(`_gridtext_bl_make_text_box`, label, gp, voff_pt)
}

bl_make_raster_box <- function(image, width_pt = 0, height_pt = 0, width_policy = "native", height_policy = "native", respect_aspect = TRUE, interpolate = TRUE, dpi = 150, gp = NULL) {
    .Call(`_gridtext_bl_make_raster_box`, image, width_pt, height_pt, width_policy, height_policy, respect_aspect, interpolate, dpi, gp)
}

bl_make_vbox <- function(node_list, width_pt = 0, hjust = 0, vjust = 1, width_policy = "native") {
    .Call(`_gridtext_bl_make_vbox`, node_list, width_pt, hjust, vjust, width_policy)
}

bl_make_regular_space_glue <- function(gp, stretch_ratio = 0.5, shrink_ratio = 0.333333) {
    .Call(`_gridtext_bl_make_regular_space_glue`, gp, stretch_ratio, shrink_ratio)
}

bl_make_forced_break_penalty <- function() {
    .Call(`_gridtext_bl_make_forced_break_penalty`)
}

bl_make_never_break_penalty <- function() {
    .Call(`_gridtext_bl_make_never_break_penalty`)
}

bl_box_width <- function(node) {
    .Call(`_gridtext_bl_box_width`, node)
}

bl_box_height <- function(node) {
    .Call(`_gridtext_bl_box_height`, node)
}

bl_box_ascent <- function(node) {
    .Call(`_gridtext_bl_box_ascent`, node)
}

bl_box_descent <- function(node) {
    .Call(`_gridtext_bl_box_descent`, node)
}

bl_box_voff <- function(node) {
    .Call(`_gridtext_bl_box_voff`, node)
}

bl_calc_layout <- function(node, width_pt = 0, height_pt = 0) {
    invisible(.Call(`_gridtext_bl_calc_layout`, node, width_pt, height_pt))
}

bl_place <- function(node, x_pt, y_pt) {
    invisible(.Call(`_gridtext_bl_place`, node, x_pt, y_pt))
}

bl_render <- function(node, x_pt = 0, y_pt = 0) {
    .Call(`_gridtext_bl_render`, node, x_pt, y_pt)
}

grid_renderer <- function() {
    .Call(`_gridtext_grid_renderer`)
}

grid_renderer_text <- function(gr, label, x, y, gp) {
    invisible(.Call(`_gridtext_grid_renderer_text`, gr, label, x, y, gp))
}

grid_renderer_text_details <- function(label, gp) {
    .Call(`_gridtext_grid_renderer_text_details`, label, gp)
}

grid_renderer_raster <- function(gr, image, x, y, width, height, interpolate = TRUE) {
    invisible(.Call(`_gridtext_grid_renderer_raster`, gr, image, x, y, width, height, interpolate))
}

grid_renderer_rect <- function(gr, x, y, width, height, gp, r = 0L) {
    invisible(.Call(`_gridtext_grid_renderer_rect`, gr, x, y, width, height, gp, r))
}

grid_renderer_collect_grobs <- function(gr) {
    .Call(`_gridtext_grid_renderer_collect_grobs`, gr)
}

unit_pt <- function(x) {
    .Call(`_gridtext_unit_pt`, x)
}

gpar_empty <- function() {
    .Call(`_gridtext_gpar_empty`)
}

text_grob <- function(label, x_pt = 0L, y_pt = 0L, gp = NULL, name = NULL) {
    .Call(`_gridtext_text_grob`, label, x_pt, y_pt, gp, name)
}

raster_grob <- function(image, x_pt = 0L, y_pt = 0L, width_pt = 0L, height_pt = 0L, interpolate = TRUE, gp = NULL, name = NULL) {
    .Call(`_gridtext_raster_grob`, image, x_pt, y_pt, width_pt, height_pt, interpolate, gp, name)
}

rect_grob <- function(x_pt = 0L, y_pt = 0L, width_pt = 0L, height_pt = 0L, gp = NULL, name = NULL) {
    .Call(`_gridtext_rect_grob`, x_pt, y_pt, width_pt, height_pt, gp, name)
}

roundrect_grob <- function(x_pt = 0L, y_pt = 0L, width_pt = 0L, height_pt = 0L, r_pt = 5L, gp = NULL, name = NULL) {
    .Call(`_gridtext_roundrect_grob`, x_pt, y_pt, width_pt, height_pt, r_pt, gp, name)
}

set_grob_coords <- function(grob, x, y) {
    .Call(`_gridtext_set_grob_coords`, grob, x, y)
}

